# Module with helper plotting functions
# for ERIS 2017 Polarization Tutorial 
# (Marti-Vidal, Oct. 2017)

import pylab as pl
import numpy as np
import os, sys
from matplotlib.collections import LineCollection
from taskinit import gentools
tb = gentools(['tb'])[0]
ia = gentools(['ia'])[0]





def getData(data,column):
  if column=='amp':
    return [np.abs(data), 'Amplitude (Norm)']
  elif column=='phase':
    return [180./np.pi*np.angle(data), 'Phase (deg.)']
  elif column=='real':
    return [np.real(data),'Real part']
  elif column=='imag':
    return [np.imag(data), 'Imaginary part']
  elif column=='cross-phase':
    return [[180./np.pi*np.angle(data[0]/data[1])], 'X/Y Cross-Phase (deg.)']




def plot_B(table1,table2='',spw1=0, spw2=0,entry1=0,entry2=0,toplot=['amp','phase']):

  """ Plots the spectral window spw1 of the bandpass table1.
      Generates one figure per antenna and stores 
      figures in the *.PLOTS directory. If the bandpass table
      contains more than one entry per antenna, entry1 selects
      which entry to plot.

      If table2 is another bandpass table, the two tables are
      plotted together, for comparison. 

      Each figure has as many subplots as elements in the "toplot"
      list. The elements can be: 'amp', 'phase', 'real', 'imag'
      or 'cross-phase'

  """

  print 'PLOT BANDPASS'

  success = tb.open('%s/ANTENNA'%table1)
  if not success:
    raise Exception("WRONG CASA TABLE %s"%table1)

  ANTS1 = list(tb.getcol('NAME'))
  tb.close()

  tb.open(table1)
  BP1 = tb.getcol('CPARAM')
  FG1 = tb.getcol('FLAG')
  NPOL1 = np.shape(BP1)[0]
  P1 = {1:'T', 2: 'X'}[NPOL1]
  ANTSOL1 = tb.getcol('ANTENNA1')


  SPWS1 = tb.getcol('SPECTRAL_WINDOW_ID')==spw1
  if np.sum(SPWS1)==0:
    raise Exception("WRONG SPECTRAL WINDOW %i FOR %s"%(spw1,table1))
  tb.close()

  success = tb.open('%s/SPECTRAL_WINDOW'%table1)
  FRQ1 = tb.getcell('CHAN_FREQ',spw1)/1.e9
  tb.close()

  filename = os.path.basename(table1).split('.')
  Name1 = filename[-1]

  if len(table2)==0:
    isTable2 = False
    destdir = '%s.PLOTS'%table1

  else:

    isTable2 = True

    success = tb.open('%s/ANTENNA'%table2)
    if not success:
      raise Exception("WRONG CASA TABLE %s"%table2)

    filename2 = os.path.basename(table2).split('.')
    Name2 = filename2[-1]
    if Name1==Name2:
      Name1 = 'B 1'
      Name2 = 'B 2'
      destdir = '%s_VS_%s.COMPARE_PLOTS'%(table1,table2)
    else:
      destdir = '%s_VS_%s.COMPARE_PLOTS'%(Name1,Name2)


    ANTS2 = list(tb.getcol('NAME'))
    tb.close()

    tb.open(table2)
    BP2 = tb.getcol('CPARAM')
    FG2 = tb.getcol('FLAG')
    NPOL2 = np.shape(BP2)[0]
    P2 = {1:'T', 2: 'X'}[NPOL2]

    ANTSOL2 = tb.getcol('ANTENNA1')

    NCHAN2 = np.shape(BP1)[1]

    SPWS2 = tb.getcol('SPECTRAL_WINDOW_ID')==spw2
    if np.sum(SPWS2)==0:
      raise Exception("WRONG SPECTRAL WINDOW %i FOR %s"%(spw2,table2))
    tb.close()

    success = tb.open('%s/SPECTRAL_WINDOW'%table2)
    FRQ2 = tb.getcell('CHAN_FREQ',spw2)/1.e9
    tb.close()

#    destdir = '%s_%s.COMPARE_PLOTS'%(table1,table2)

  fig = pl.figure()

  if not os.path.exists(destdir):
    os.system('mkdir %s'%destdir)


  for a1i,ant1 in enumerate(ANTS1):

    Pmin = [1000. for i in toplot] 
    Pmax = [-1000. for i in toplot] 

    fig.clf()
    subplots = [fig.add_subplot(len(toplot),1,1)]

    for i in range(1,len(toplot)):
      subplots.append(fig.add_subplot(len(toplot),1,i+1,sharex=subplots[0]))

    fig.subplots_adjust(hspace = 0.05,wspace=0.05)
    for i in range(len(toplot)-1):
      pl.setp(subplots[i].get_xticklabels(),'visible',False)

  #  for i in range(len(toplot)):
  #    subplots[i].cla()

    Entry1 = np.where((ANTSOL1==a1i)*SPWS1)[0]
    Nentry = len(Entry1)
    if Nentry>1:
      print 'INFO: THERE ARE %i ENTRIES FOR %s IN BP1'%(Nentry,ant1)
    if entry1>=Nentry:
      raise Exception('ERROR: ENTRY %i IN BP1 DOES NOT EXIST!'%entry1) 

    BPlot1 = BP1[:,:,Entry1[entry1]]
    Flag1 = FG1[:,:,Entry1[entry1]]
    NFlag1 = np.logical_not(Flag1)

    for i in range(len(toplot)):

      plotData, axisname = getData(BPlot1,toplot[i])
      Pmin[i] = np.min([np.min(plotData),Pmin[i]])
      Pmax[i] = np.max([Pmax[i],np.max(plotData)])
   #   print 'PLOTDATA: ',np.max(plotData), Pmax[i]
      subplots[i].plot(FRQ1[NFlag1[0,:]],plotData[0][NFlag1[0,:]],'-r',label='%s - %s'%(Name1,P1),linewidth=2)
      subplots[i].plot(FRQ1[Flag1[0,:]],plotData[0][Flag1[0,:]],'xk')

      if NPOL1==2 and toplot[i]!='cross-phase':
        subplots[i].plot(FRQ1[NFlag1[1,:]],plotData[1,NFlag1[1,:]],'-b',label='%s - Y'%Name1,linewidth=2)
        subplots[i].plot(FRQ1[Flag1[1,:]],plotData[1,Flag1[1,:]],'xk')

      subplots[i].set_ylabel(axisname)

    subplots[-1].set_xlabel('Frequency (GHz)')


    if isTable2:

     if ant1 in ANTS2:
      a2i = ANTS2.index(ant1)

      Entry2 = np.where((ANTSOL2==a2i)*SPWS2)[0]
      Nentry = len(Entry2)
      if Nentry>1:
        print 'INFO: THERE ARE %i ENTRIES FOR %s IN BP2'%(Nentry,ant1)
      if entry2>=Nentry:
        raise Exception('ERROR: ENTRY %i IN BP2 DOES NOT EXIST!'%entry2) 


      BPlot2 = BP2[:,:,Entry2[entry2]]
      Flag2 = FG2[:,:,Entry2[entry2]]
      NFlag2 = np.logical_not(Flag2)

      for i in range(len(toplot)):

        plotData, axisname = getData(BPlot2,toplot[i])
        Pmin[i] = np.min([Pmin[i],np.min(plotData)])
        Pmax[i] = np.max([Pmax[i],np.max(plotData)])
        subplots[i].plot(FRQ2[NFlag2[0,:]],plotData[0][NFlag2[0,:]],'--g',label='%s - %s'%(Name2,P2),linewidth=2)
        subplots[i].plot(FRQ2[Flag2[0,:]],plotData[0][Flag2[0,:]],'xk')

        if NPOL2==2 and toplot[i]!='cross-phase':
          subplots[i].plot(FRQ2[NFlag2[0,:]],plotData[1,NFlag2[1,:]],'--m',label='%s - Y'%Name2,linewidth=2)
          subplots[i].plot(FRQ2[Flag2[1,:]],plotData[1,Flag2[1,:]],'xk')



     else:
      print 'WARNING: ANTENNA %s NOT IN BP2!'%ant1


    for i in range(len(toplot)):
      PlotMin = Pmin[i] ; PlotMax = Pmax[i]
      if PlotMin==PlotMax:
        PlotMin = 0. ; PlotMax = 1.

  #    print PlotMin, PlotMax
      if toplot[i]=='amp':
        subplots[i].set_ylim((PlotMin - (PlotMax-PlotMin)*0.1, PlotMax + (PlotMax-PlotMin)*0.25))
      elif toplot[i]=='phase' and np.abs(PlotMin-PlotMax)<0.1:
        subplots[i].set_ylim((PlotMin - 10., PlotMax + 10.))
      else:
        subplots[i].set_ylim((PlotMin - (PlotMax-PlotMin)*0.1, PlotMax + (PlotMax-PlotMin)*0.1))



    subplots[0].legend(numpoints=1,ncol = 2,prop={'size':8})

    pl.suptitle('ANTENNA %s'%ant1)
    pl.savefig('%s/%s_spw%i.png'%(destdir,ant1,spw1))

  print 'DONE!'






  


def zeroRefant(table1,refant=0):

  success = tb.open('%s/ANTENNA'%table1)
  if not success:
    raise Exception("ERROR! Invalid CASA Table %s"%table1)

  antlist = list(tb.getcol('NAME'))
  try:
    refantidx = int(refant)
  except:
    refantidx = antlist.index(refant)


  tb.open(table1,nomodify=False)
  timearr = tb.getcol('TIME')
  calant = tb.getcol('ANTENNA2')
  solant = tb.getcol('ANTENNA1')
  flagant = tb.getcol('FLAG')
  ifsol = tb.getcol('SPECTRAL_WINDOW_ID')
  allifs = list(set(ifsol))
  alltimes = list(set(timearr))
  gains = tb.getcol('CPARAM')
  for ti in alltimes:
    is2 = (timearr==ti)  #*(calant==REFANT_IDX)
    is22 = is2*(solant==refantidx)
    for IFi in allifs:
      isifref = is22*(ifsol==IFi)
      isifcal = is2*(ifsol==IFi)
      if np.sum(isifref)>0:
        gains[:,:,isifcal] /= gains[:,:,isifref]

  tb.putcol('CPARAM',gains)
  tb.close()










def plot_G(table1, table2='', spw1=0, spw2=0,toplot=['amp','phase']):

  """ Plots the spectral window spw1 of the gains in table1.
      Generates one figure per antenna and stores 
      figures in the *.PLOTS directory.

      If table2 is another gain table, the two tables are
      plotted together, for comparison. 

      Each figure has as many subplots as elements in the "toplot"
      list. The elements can be: 'amp', 'phase', 'real' or 'imag'.

  """

  print 'PLOT GAIN VS. TIME'

#  print 'PLOTTING %s/ANTENNA'%table1
  success = tb.open('%s/ANTENNA'%table1)
  if not success:
    raise Exception("WRONG CASA TABLE %s"%table1)

  ANTS1 = list(tb.getcol('NAME'))
  tb.close()

  tb.open(table1)
  G1 = tb.getcol('CPARAM')
  F1 = tb.getcol('FLAG')
  NPOL1 = np.shape(G1)[0]
  P1 = {1:'T', 2: 'X'}[NPOL1]
  ANTSOL1 = tb.getcol('ANTENNA1')

  TIME1 = tb.getcol('TIME')/86400.
  T0 = np.floor(np.min(TIME1))
  TIME1 -= T0
  TIME1 *= 24.

  SPWS1 = tb.getcol('SPECTRAL_WINDOW_ID')==spw1
  if np.sum(SPWS1)==0:
    raise Exception("WRONG SPECTRAL WINDOW %i FOR %s"%(spw1,table1))
  tb.close()

  filename = os.path.basename(table1).split('.')
  Name1 = filename[-1]

  if len(table2)==0:
    isTable2 = False
    destdir = '%s.PLOTS'%table1

  else:
    isTable2 = True

    success = tb.open('%s/ANTENNA'%table2)
    if not success:
      raise Exception("WRONG CASA TABLE %s"%table2)

    filename2 = os.path.basename(table2).split('.')
    Name2 = filename2[-1]
    if Name1==Name2:
      Name1 = 'G 1'
      Name2 = 'G 2'
      destdir = '%s_VS_%s.COMPARE_PLOTS'%(table1,table2)
    else:
      destdir = '%s_VS_%s.COMPARE_PLOTS'%(Name1,Name2)


    ANTS2 = list(tb.getcol('NAME'))
    tb.close()

    tb.open(table2)
    G2 = tb.getcol('CPARAM')
    F2 = tb.getcol('FLAG')
    NPOL2 = np.shape(G2)[0]
    P2 = {1:'T', 2: 'X'}[NPOL2]
    ANTSOL2 = tb.getcol('ANTENNA1')
    TIME2 = 24.*(tb.getcol('TIME')/86400. - T0)

    SPWS2 = tb.getcol('SPECTRAL_WINDOW_ID')==spw2
    if np.sum(SPWS2)==0:
      raise Exception("WRONG SPECTRAL WINDOW %i FOR %s"%(spw2,table2))
    tb.close()


  if not os.path.exists(destdir):
    os.system('mkdir %s'%destdir)


  fig = pl.figure()


  for a1i,ant1 in enumerate(ANTS1):
    Pmin = [1000. for i in toplot] 
    Pmax = [-1000. for i in toplot] 

    fig.clf()

    subplots = [fig.add_subplot(len(toplot),1,1)]

    for i in range(1,len(toplot)):
      subplots.append(fig.add_subplot(len(toplot),1,i+1,sharex=subplots[0]))

    fig.subplots_adjust(hspace = 0.05,wspace=0.05)
    for i in range(len(toplot)-1):
      pl.setp(subplots[i].get_xticklabels(),'visible',False)




    Entry1 = np.where((ANTSOL1==a1i)*SPWS1)[0]
    GPlot1X = G1[0,0,Entry1]
    Flag1X = F1[0,0,Entry1]
    NFlag1X = np.logical_not(Flag1X)

    if NPOL1==2:
      GPlot1Y = G1[1,0,Entry1]
      Flag1Y = F1[0,0,Entry1]
      NFlag1Y = np.logical_not(Flag1Y)

    TPlot1 = TIME1[Entry1]

    for i in range(len(toplot)):

      if toplot[i]=='cross-phase':
        plotDataX, axisname = getData([GPlot1X,GPlot1Y],toplot[i])
        plotDataX = plotDataX[0]
      else:
        plotDataX, axisname = getData(GPlot1X,toplot[i])

      if np.sum(NFlag1X)>0:
        Pmin[i] = np.min([np.min(plotDataX[NFlag1X]),Pmin[i]])
        Pmax[i] = np.max([np.max(plotDataX[NFlag1X]),Pmax[i]])

      if NPOL1==2 and toplot[i]!='cross-phase':
        plotDataY, axisname = getData(GPlot1Y,toplot[i])
        if np.sum(NFlag1Y)>0:
          Pmin[i] = np.min([np.min(plotDataY[NFlag1Y]),Pmin[i]])
          Pmax[i] = np.max([np.max(plotDataY[NFlag1Y]),Pmax[i]])

      subplots[i].plot(TPlot1[NFlag1X],plotDataX[NFlag1X],'or',label='%s - %s'%(Name1,P1),markeredgewidth=0.0)
      subplots[i].plot(TPlot1[Flag1X],plotDataX[Flag1X],'xk')

      if NPOL1==2 and toplot[i]!='cross-phase':
        subplots[i].plot(TPlot1[NFlag1Y],plotDataY[NFlag1Y],'ob',label='%s - Y'%Name1,markeredgewidth=0.0)
        subplots[i].plot(TPlot1[Flag1Y],plotDataY[Flag1Y],'xk')


      subplots[i].set_ylabel(axisname)


    if isTable2:

     if ant1 in ANTS2:
      a2i = ANTS2.index(ant1)

      Entry2 = np.where((ANTSOL2==a2i)*SPWS2)[0]

      GPlot2X = G2[0,0,Entry2]
      Flag2X = F2[0,0,Entry2]
      NFlag2X = np.logical_not(Flag2X)

      if NPOL2==2: 
        GPlot2Y = G2[1,0,Entry2]
        Flag2Y = F2[0,0,Entry2]
        NFlag2Y = np.logical_not(Flag2Y)


      TPlot2 = TIME2[Entry2]

      for i in range(len(toplot)):

        if toplot[i]=='cross-phase':
          plotDataX, axisname = getData([GPlot2X,GPlot2Y],toplot[i])
          plotDataX = plotDataX[0]
        else:
          plotDataX, axisname = getData(GPlot2X,toplot[i])

        if np.sum(NFlag2X)>0:
          Pmin[i] = np.min([np.min(plotDataX[NFlag2X]),Pmin[i]])
          Pmax[i] = np.max([np.max(plotDataX[NFlag2X]),Pmax[i]])

        if NPOL2==2 and toplot[i]!='cross-phase':
          plotDataY, axisname = getData(GPlot2Y,toplot[i])
          if np.sum(NFlag2Y)>0:
            Pmin[i] = np.min([np.min(plotDataY[NFlag2Y]),Pmin[i]])
            Pmax[i] = np.max([np.max(plotDataY[NFlag2Y]),Pmax[i]])


        subplots[i].plot(TPlot2[NFlag2X],plotDataX[NFlag2X],'sg',label='%s - %s'%(Name2,P2),markeredgewidth=0.0)
        subplots[i].plot(TPlot2[Flag2X],plotDataX[Flag2X],'xk')

        if NPOL2==2 and toplot[i]!='cross-phase':
          subplots[i].plot(TPlot2[NFlag2X],plotDataY[NFlag2Y],'sm',label='%s - Y'%Name2,markeredgewidth=0.0)
          subplots[i].plot(TPlot2[Flag2Y],plotDataY[Flag2Y],'xk')


     else:
      print 'WARNING: ANTENNA %s NOT IN G2!'%ant1


    subplots[-1].set_xlabel('UT (h)')


    for i in range(len(toplot)):
      PlotMin = Pmin[i] ; PlotMax = Pmax[i]
      if PlotMin==PlotMax:
        PlotMin = 0. ; PlotMax = 1.

      if toplot[i]=='amp':
        subplots[i].set_ylim((PlotMin - (PlotMax-PlotMin)*0.1, PlotMax + (PlotMax-PlotMin)*0.25 ))
      elif toplot[i]=='phase' and np.abs(PlotMax-PlotMin)<0.1:
        subplots[i].set_ylim((PlotMin - 10., PlotMax + 10. ))
      else:
        subplots[i].set_ylim((PlotMin - (PlotMax-PlotMin)*0.1, PlotMax + (PlotMax-PlotMin)*0.1 ))

    subplots[0].legend(numpoints=1,ncol=2,prop={'size':8})

    pl.suptitle('ANTENNA %s'%ant1)
    pl.savefig('%s/%s_spw%i.png'%(destdir,ant1,spw1))

  print 'DONE!'










def finetune_G(table1,XphaseAdd=0.,YphaseAdd=0.,XampProd=1.,YampProd=1.,
               XrealAdd=0., YrealAdd=0., XimagAdd=0., YimagAdd=0., antennas=[]):
  """ Modify the gains in table1:

      1.- Adds a phase of XphaseAdd degrees to all gains in X.
      2.- Adds a phase of YphaseAdd degrees to all gains in Y.
      3.- Multiplies by XampProd all gains in X.
      4.- Multiplies by YampProd all gains in Y.
      5.- Adds XrealAdd to the real part of the X gains.
      6.- Adds YrealAdd to the real part of the Y gains.
      7.- Adds XimagAdd to the imag part of the X gains.
      8.- Adds YimagAdd to the imag part of the Y gains.

      if antennas=[] (default), modify all entries. If a list of antenna names,
      restrict the gain changes to the antennas in the list. """


  success = tb.open('%s/ANTENNA'%table1)
  if not success:
    raise Exception("ERROR! Not a valid CASA table!")

  antnames = list(tb.getcol('NAME'))
  tb.close()

  if len(antennas)==0:
    antidx = range(len(antnames))
  else:
    antidx = []
    for ant in antennas:
      if ant in antnames:
        antidx.append(antnames.index(ant))
      else:
        print 'WARNING: ANTENNA %s NOT IN TABLE'%ant
    if len(antidx)==0:
      raise Exception("ERROR! NONE OF THE ANTENNAS WAS FOUND IN THE TABLE!")

  tb.open(table1,nomodify=False)

  if 'CPARAM' in tb.colnames():
    data = tb.getcol('CPARAM')
    isC = True
  else:
    data = tb.getcol('FPARAM')
    isC = False

  ants = tb.getcol('ANTENNA1')

  for ant in antidx:
    sys.stdout.write('\rModifying gains for %s'%ant)
    sys.stdout.flush()
    mask = ants==ant
   
    if isC:
      data[0,:,mask] *= XampProd*np.exp(1.j*XphaseAdd*np.pi/180.)
      data[1,:,mask] *= YampProd*np.exp(1.j*YphaseAdd*np.pi/180.)
      data[0,:,mask] += XrealAdd + 1.j*XimagAdd 
      data[1,:,mask] += YrealAdd + 1.j*YimagAdd 
    else:
      data[0,:,mask] *= XampProd
      data[1,:,mask] *= YampProd
      data[0,:,mask] += XrealAdd
      data[1,:,mask] += YrealAdd 


  print '\n\n DONE!'

  if isC:
    tb.putcol('CPARAM',data)
  else:
    tb.putcol('FPARAM',data)

  tb.close()










def plotPolImage(image,residual='',sigmaCut=3.0, nContour=10, channel=0, plotrange=[], NpixPol=4,polLength=0.1):

  """ Plot an interactive image of a full-polarization image (linear pol + V Stokes). If the 
      image is a cube, only one channel is plotted ('channel' keyword).

      - sigmaCut: cutoff of the linear-polarization image (in units of TMS of residual image)
      - Number of contours for Stokes I (log distributed).
      - plotrange: [RAmin, RAmax, Decmin, Decmax] (in arcsec, referred to image center)
      - NpixPol: draw pol. lines every NpixPol pixels.
      - polLength: length of pol. lines (in arcsec).
  """

  

  unit = {'rad':180./np.pi*3600,'deg':3600.,'arcsec':1.}

  try:
    ia.open(image)
  except:
    raise Exception("ERROR! Image is not a valid CASA image")

  summ = ia.summary()

  AxName = list(summ['axisnames'])
  RA = AxName.index('Right Ascension')
  Dec = AxName.index('Declination')
  Pol = AxName.index('Stokes')
  Freq = AxName.index('Frequency')

  Deltas = summ['incr']/2.
  Un = summ['axisunits']
  Shape = summ['shape']
  if type(channel) is not int or channel<0 or channel >= Shape[Freq]:
    raise Exception("ERROR! Bad channel number! Check image dimensions (%i channels)"%Shape[Freq])

  Extent = [-Deltas[RA]*Shape[RA]*unit[Un[RA]],Deltas[RA]*Shape[RA]*unit[Un[RA]],-Deltas[Dec]*Shape[Dec]*unit[Un[Dec]],Deltas[Dec]*Shape[Dec]*unit[Un[Dec]]]

  if len(plotrange)==0:
    plotrange = Extent
  elif len(plotrange)!= 4:
    raise Exception("ERROR! Invalid plot range!")

  Data = np.transpose(ia.getchunk(),[RA,Dec,Pol,Freq])[:,:,:,channel]

  ia.close()

  if len(residual)>0 and sigmaCut>0.0:
    try:
      ia.open(residual)
    except:
      raise Exception("ERROR! Residual is not a valid CASA image")

  resdata = np.transpose(ia.getchunk(),[RA,Dec,Pol,Freq])
  RMS = np.std(resdata[:,:,0,channel])
  RMSV = np.std(resdata[:,:,3,channel])
  ResCut = RMS*sigmaCut

  ia.close()

  fig = pl.figure(figsize=(14,5))

  Pimg = fig.add_subplot(121,aspect='equal')
  Vimg = fig.add_subplot(122,aspect='equal',sharex=Pimg,sharey=Pimg)

  fig.subplots_adjust(left=0.08,right=0.85,wspace=0.5)

  S1CB = fig.add_axes([0.4,0.05,0.02,0.85])
  S2CB = fig.add_axes([0.9,0.05,0.02,0.85])

  mData = np.sqrt(Data[:,:,1]**2. + Data[:,:,2]**2.)

  mData[mData[:,:]<ResCut]=np.nan

  mData *= 1000.

  PhiData = 0.5*np.arctan2(Data[:,:,2],Data[:,:,1])

  PhiData[np.isnan(mData)] = np.nan

  xc = np.linspace(Extent[0],Extent[1],Shape[RA])
  yc = np.linspace(Extent[2],Extent[3],Shape[Dec])
  xyc = np.meshgrid(yc,xc)



  clevs = np.logspace(np.log10(ResCut),np.log10(np.max(Data[:,:,0])),nContour)

  VIm = Vimg.imshow(1000.*np.transpose(Data[:,:,3]),origin='lower',aspect='equal',extent=Extent,cmap='Blues',picker=True)
  Vcb = pl.colorbar(VIm,cax=S2CB)
  Vcb.set_label('V Stokes (mJy/beam)')

  Vimg.set_title('Circ. Polarization')

  pIm = Pimg.imshow(np.transpose(mData),origin='lower',aspect='equal',vmin=0.0,extent=Extent,cmap='cool',picker=True)
  Pimg.contour(xyc[1],xyc[0],Data[:,:,0],levels=clevs,colors='k')
  Vimg.contour(xyc[1],xyc[0],Data[:,:,0],levels=clevs,colors='k')

  Pimg.set_title('Lin. Polarization')

  PolLines = np.zeros(np.shape(Data[:,:,0]),dtype=np.bool)
  PolLines[::NpixPol,::NpixPol] = True
  PolLines[np.isnan(mData)] = False

  PolVec = np.where(PolLines)

  Kf = polLength
  lines = []
  for pi in range(len(PolVec[0])):
    Yp = np.cos(PhiData[PolVec[0][pi],PolVec[1][pi]])
    Xp = np.sin(PhiData[PolVec[0][pi],PolVec[1][pi]])
    X0 = xc[PolVec[0][pi]]-Xp*Kf; X1 = xc[PolVec[0][pi]]+Xp*Kf
    Y0 = yc[PolVec[1][pi]]-Yp*Kf; Y1 = yc[PolVec[1][pi]]+Yp*Kf
    lines.append(((X0,Y0),(X1,Y1)))
  lines = tuple(lines)
  PolCol = LineCollection(lines,colors='orange',linewidths=2)
  Pimg.add_collection(PolCol)

  mcb = pl.colorbar(pIm,cax=S1CB)
  mcb.set_label('m (mJy/beam)')

  Vimg.set_xlabel('Relative RA (as)')
  Vimg.set_ylabel('Relative Dec (as)')

  Pimg.set_xlabel('Relative RA (as)')
  Pimg.set_ylabel('Relative Dec (as)')

  pl.sca(Pimg)
  Pimg.set_xlim((plotrange[0],plotrange[1]))
  Pimg.set_ylim((plotrange[2],plotrange[3]))


  PeakI = np.unravel_index(np.argmax(Data[:,:,0]),np.shape(Data[:,:,0]))
  PosI = Data[PeakI[0],PeakI[1],0]
  PosP = mData[PeakI[0],PeakI[1]]/1000.
  PosPhi = 180./np.pi*PhiData[PeakI[0],PeakI[1]]
  PosV = Data[PeakI[0],PeakI[1],3]

  Pfmt = 'I:       %.2e Jy/b\nm:     %.2e Jy/b  (%.1f%%)\nEVPA: %.1f deg.\nRMS:  %.2e Jy/b' #%(PosI,PosP,PosPhi,RMS)
  Vfmt = 'V:     %.2e Jy/b\nRMS: %.2e Jy/b'
  PimgText = Pimg.text(0.05,0.8,Pfmt%(PosI,PosP,PosP/PosI*100.,PosPhi,RMS),transform=Pimg.transAxes,color='r',fontweight='bold')
  VimgText = Vimg.text(0.05,0.9,Vfmt%(PosV,RMSV),transform=Vimg.transAxes,color='r',fontweight='bold')

#  CrossP = Pimg.plot([yc[PeakI[1]]],[xc[PeakI[0]]],'+r')[0]
#  CrossV = Vimg.plot([yc[PeakI[1]]],[xc[PeakI[0]]],'+r')[0]
  CrossP = Pimg.plot([xc[PeakI[0]]],[yc[PeakI[1]]],'+r',markersize=15,markeredgewidth=3)[0]
  CrossV = Vimg.plot([xc[PeakI[0]]],[yc[PeakI[1]]],'+r',markersize=15,markeredgewidth=3)[0]


  def onPick(event):

    if event.mouseevent.inaxes in [Pimg,Vimg]:
      RAv = event.mouseevent.xdata
      Decv = event.mouseevent.ydata
      RApix = np.floor((RAv-Extent[0])/(Extent[1]-Extent[0])*Shape[RA])
      Decpix = np.floor((Decv-Extent[2])/(Extent[3]-Extent[2])*Shape[Dec])

      if RApix>0 and RApix<Shape[RA] and Decpix>0 and Decpix<Shape[Dec]:

         PosI = Data[RApix,Decpix,0]
         PosP = mData[RApix,Decpix]/1000.
         PosPhi = 180./np.pi*PhiData[RApix,Decpix]
         PosV = Data[RApix,Decpix,3]

         PimgText.set_text(Pfmt%(PosI,PosP,PosP/PosI*100.,PosPhi,RMS))
         VimgText.set_text(Vfmt%(PosV,RMSV))
         CrossP.set_data([RAv],[Decv])
         CrossV.set_data([RAv],[Decv])
         pl.draw()

  fig.canvas.mpl_connect('pick_event',onPick)

  pl.draw()
  pl.savefig('%s.fullpol_image.png'%image)

  pl.show()







def plotSchedule(msname,bandpass='',fluxcal='',phasecal='',target='',polcal=''):

  """ Plot the schedule (if bandpass, fluxcal, phasecal and target are
      provided). If more than one field shares the same intent, give 
      the list of fields as one single string of comma-separated values."""

  success = tb.open('%s/FIELD'%msname)
  if not success:
    raise Exception("ERROR! Invalid measurement set!")
  Fnames = tb.getcol('NAME')
  tb.close()


  colors = ['r','g','b','k','m','c','orange','lime']*int(len(Fnames)/8+1)
  symb = ['.','+','x','o','s','d']
  mew =  [0,   1,  1,  0,  0,  0]

  bps = [b.replace(' ','') for b in bandpass.split(',')]
  fls = [b.replace(' ','') for b in fluxcal.split(',')]
  phs = [b.replace(' ','') for b in phasecal.split(',')]
  tgs = [b.replace(' ','') for b in target.split(',')]
  pls = [b.replace(' ','') for b in polcal.split(',')]


  success = tb.open(msname)
  fi = tb.getcol('FIELD_ID')
  ti = tb.getcol('TIME')/86400.
  tb.close()

  ti -= int(np.min(ti))

  ti *= 24.

  fig = pl.figure(figsize=(14,3))
  sub = fig.add_subplot(111)

  fig.subplots_adjust(left=0.08,bottom=0.17,right=0.97,top=0.96)

  for f,field in enumerate(Fnames):
    mask = fi==f
    times = ti[mask]
    yy = np.ones(len(times))
    print field
    if field in bps:
      ss = bps.index(field)
      sub.plot(times,yy*0,symb[ss],color=colors[f],markeredgewidth=mew[ss],label=field)
    if field in fls:
      ss = fls.index(field)
      sub.plot(times,yy*1,symb[ss],color=colors[f],markeredgewidth=mew[ss],label=field)
    if field in phs:
      ss = phs.index(field)
      sub.plot(times,yy*2,symb[ss],color=colors[f],markeredgewidth=mew[ss],label=field)
    if field in pls:
      ss = pls.index(field)
      sub.plot(times,yy*3,symb[ss],color=colors[f],markeredgewidth=mew[ss],label=field)
    if field in tgs:
      ss = tgs.index(field)
      sub.plot(times,yy*4,symb[ss],color=colors[f],markeredgewidth=mew[ss],label=field)

  sub.set_yticks([0,1,2,3,4])
  sub.set_yticklabels(['Bandpass','Flux cal.','Phase cal.','Polariz. cal.', 'Target'])

  sub.set_xlabel('UT (h.)')
  sub.set_ylim((-1,7))
  
  pl.legend(numpoints=1,ncol=(len(Fnames)/2+1)) #,prop={'size':13})

  pl.savefig('%s.obslist.png'%msname)
  pl.show()



